@@ -25,6 +25,8 @@
 #' can write a statement \code{a < 5} that filters
 #' all rows in the table where values in column a
 #' are less than five.
+#' @param brief an optional, text-based description
+#' for the validation step.
 #' @param warn_count the threshold number for 
 #' individual validations returning a \code{FALSE}
 #' result before applying the \code{warn} flag.
@@ -77,10 +79,6 @@
 #' \code{l} -> logical, \code{D} -> date, \code{T} ->
 #' date time, \code{t} -> time, \code{?} -> guess, 
 #' or \code{_/-}, which skips the column.
-#' @param description an optional, text-based
-#' description for the validation step. Used primarily
-#' in the Logical Plan section of the report generated
-#' by the \code{html_summary} function.
 #' @return an agent object.
 #' @examples
 #' # Create a simple data frame with
@@ -111,11 +109,14 @@
 #' #> [1] TRUE
 #' @importFrom tibble tibble
 #' @importFrom dplyr bind_rows
-#' @importFrom rlang enquo UQ
+#' @importFrom rlang enquo expr_text
+#' @importFrom stringr str_replace_all
 #' @export col_vals_null
 
 col_vals_null <- function(agent,
                           column,
+                          preconditions = NULL,
+                          brief = NULL,
                           warn_count = 1,
                           notify_count = NULL,
                           warn_fraction = NULL,
@@ -125,20 +126,35 @@ col_vals_null <- function(agent,
                           creds_file = NULL,
                           initial_sql = NULL,
                           file_path = NULL,
-                          col_types = NULL,
-                          preconditions = NULL,
-                          description = NULL) {
+                          col_types = NULL) {
   
-  column <- rlang::enquo(column)
-  column <- (rlang::UQ(column) %>% paste())[2]
+  # Get the column name
+  column <- 
+    rlang::enquo(column) %>%
+    rlang::expr_text() %>%
+    stringr::str_replace_all("~", "") %>%
+    stringr::str_replace_all("\"", "'")
   
-  preconditions <- rlang::enquo(preconditions)
-  preconditions <- (rlang::UQ(preconditions) %>% paste())[2]
+  # Get the preconditions
+  preconditions <- 
+    rlang::enquo(preconditions) %>%
+    rlang::expr_text() %>%
+    stringr::str_replace_all("~", "") %>%
+    stringr::str_replace_all("\"", "'")
   
-  if (preconditions == "NULL") {
+  if (length(preconditions) == 0) {
     preconditions <- NULL
   }
   
+  if (is.null(brief)) {
+    
+    brief <-
+      create_autobrief(
+        agent = agent,
+        assertion_type = "col_vals_null",
+        column = column)
+  }
+  
   # If "*" is provided for `column`, select all
   # table columns for this verification
   if (column[1] == "all_cols()") {
@@ -151,11 +167,12 @@ col_vals_null <- function(agent,
       agent = agent,
       assertion_type = "col_vals_null",
       column = column,
+      preconditions = preconditions,
+      brief = brief,
       warn_count = warn_count,
       notify_count = notify_count,
       warn_fraction = warn_fraction,
       notify_fraction = notify_fraction,
-      preconditions = preconditions,
       tbl_name = ifelse(is.null(tbl_name), as.character(NA), tbl_name),
       db_type = ifelse(is.null(db_type), as.character(NA), db_type),
       creds_file = ifelse(is.null(creds_file), as.character(NA), creds_file),
@@ -163,9 +180,9 @@ col_vals_null <- function(agent,
       file_path = ifelse(is.null(file_path), as.character(NA), file_path),
       col_types = ifelse(is.null(col_types), as.character(NA), col_types))
   
-  # If no `description` provided, set as `NA`
-  if (is.null(description)) {
-    description <- as.character(NA)
+  # If no `brief` provided, set as NA
+  if (is.null(brief)) {
+    brief <- as.character(NA)
   }
   
   # Place the validation step in the logical plan
@@ -175,7 +192,7 @@ col_vals_null <- function(agent,
       tibble::tibble(
         component_name = "col_vals_null",
         parameters = as.character(NA),
-        description = description))
+        brief = brief))
   
-  return(agent)
+  agent
 }
